"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const AstronomicalObject_1 = __importDefault(require("../astronomicalObject/AstronomicalObject"));
const epoch_1 = require("../constants/epoch");
const apparentCoordinateCalc_1 = require("../coordinates/calculations/apparentCoordinateCalc");
const coordinateCalc_1 = require("../coordinates/calculations/coordinateCalc");
const precessionCalc_1 = require("../coordinates/calculations/precessionCalc");
const starCalc_1 = require("./calculations/starCalc");
class Star extends AstronomicalObject_1.default {
    constructor(equatorialCoords, toi, properMotion = { rightAscension: 0, declination: 0 }, referenceEpoch = epoch_1.EPOCH_J2000) {
        super(toi, 'star');
        this.equatorialCoords = equatorialCoords;
        this.properMotion = properMotion;
        this.referenceEpoch = referenceEpoch;
    }
    getHeliocentricEclipticSphericalJ2000Coordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getGeocentricEclipticSphericalJ2000Coordinates();
        });
    }
    getHeliocentricEclipticSphericalDateCoordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getGeocentricEclipticSphericalDateCoordinates();
        });
    }
    getHeliocentricEclipticRectangularJ2000Coordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getGeocentricEclipticRectangularJ2000Coordinates();
        });
    }
    getHeliocentricEclipticRectangularDateCoordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getGeocentricEclipticRectangularDateCoordinates();
        });
    }
    getGeocentricEclipticRectangularJ2000Coordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            const coords = yield this.getGeocentricEclipticSphericalJ2000Coordinates();
            return (0, coordinateCalc_1.spherical2rectangular)(coords);
        });
    }
    getGeocentricEclipticRectangularDateCoordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            const coords = yield this.getGeocentricEclipticSphericalDateCoordinates();
            return (0, coordinateCalc_1.spherical2rectangular)(coords);
        });
    }
    getGeocentricEclipticSphericalJ2000Coordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            const coords = yield this.getGeocentricEquatorialSphericalJ2000Coordinates();
            return (0, coordinateCalc_1.equatorialSpherical2eclipticSpherical)(coords, this.T);
        });
    }
    getGeocentricEclipticSphericalDateCoordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            const coords = yield this.getGeocentricEquatorialSphericalDateCoordinates();
            return (0, coordinateCalc_1.equatorialSpherical2eclipticSpherical)(coords, this.T);
        });
    }
    getGeocentricEquatorialSphericalJ2000Coordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.resolve(this.equatorialCoords);
        });
    }
    getGeocentricEquatorialSphericalDateCoordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            const coords = (0, starCalc_1.correctProperMotion)(this.equatorialCoords, this.properMotion, this.jd, this.referenceEpoch);
            return Promise.resolve((0, precessionCalc_1.correctPrecessionForEquatorialCoordinates)(coords, this.jd, this.referenceEpoch));
        });
    }
    getApparentGeocentricEclipticSphericalCoordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            let coords = yield this.getGeocentricEclipticSphericalDateCoordinates();
            coords = (0, apparentCoordinateCalc_1.correctEffectOfAberration)(coords, this.T);
            return (0, apparentCoordinateCalc_1.correctEffectOfNutation)(coords, this.T);
        });
    }
    getApparentGeocentricEquatorialSphericalCoordinates() {
        return __awaiter(this, void 0, void 0, function* () {
            const coords = yield this.getApparentGeocentricEclipticSphericalCoordinates();
            return (0, coordinateCalc_1.eclipticSpherical2equatorialSpherical)(coords, this.T);
        });
    }
}
exports.default = Star;
